@@ -1,10 +1,10 @@
-/* $Id: initterm-r0drv.cpp 83687 2013-02-08 12:50:49Z fmehnert $ */
+/* $Id: initterm-r0drv.cpp 118412 2017-10-17 14:26:02Z bird $ */
 /** @file
  * IPRT - Initialization & Termination, R0 Driver, Common.
  */
 
 /*
- * Copyright (C) 2006-2011 Oracle Corporation
+ * Copyright (C) 2006-2017 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -25,9 +25,9 @@
  */
 
 
-/*******************************************************************************
-*   Header Files                                                               *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Header Files                                                                                                                 *
+*********************************************************************************************************************************/
 #include <iprt/initterm.h>
 #include "internal/iprt.h"
 
@@ -42,12 +42,13 @@
 #endif
 
 #include "internal/initterm.h"
+#include "internal/mem.h"
 #include "internal/thread.h"
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
+/*********************************************************************************************************************************
+*   Global Variables                                                                                                             *
+*********************************************************************************************************************************/
 /** Count of current IPRT users.
  * In ring-0 several drivers / kmods / kexts / wossnames may share the
  * same runtime code. So, we need to keep count in order not to terminate
@@ -64,20 +65,32 @@ static int32_t volatile g_crtR0Users = 0;
 RTR0DECL(int) RTR0Init(unsigned fReserved)
 {
     int rc;
-    Assert(fReserved == 0);
+    uint32_t cNewUsers;
+    Assert(fReserved == 0); RT_NOREF_PV(fReserved);
+#ifndef RT_OS_SOLARIS       /* On Solaris our thread preemption information is only obtained in rtR0InitNative().*/
     RT_ASSERT_PREEMPTIBLE();
+#endif
 
     /*
      * The first user initializes it.
      * We rely on the module loader to ensure that there are no
      * initialization races should two modules share the IPRT.
      */
-    if (ASMAtomicIncS32(&g_crtR0Users) != 1)
-        return VINF_SUCCESS;
+    cNewUsers = ASMAtomicIncS32(&g_crtR0Users);
+    if (cNewUsers != 1)
+    {
+        if (cNewUsers > 1)
+            return VINF_SUCCESS;
+        ASMAtomicDecS32(&g_crtR0Users);
+        return VERR_INTERNAL_ERROR_3;
+    }
 
     rc = rtR0InitNative();
     if (RT_SUCCESS(rc))
     {
+#ifdef RTR0MEM_WITH_EF_APIS
+        rtR0MemEfInit();
+#endif
         rc = rtThreadInit();
         if (RT_SUCCESS(rc))
         {
@@ -96,6 +109,9 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
 #endif
             rtThreadTerm();
         }
+#ifdef RTR0MEM_WITH_EF_APIS
+        rtR0MemEfTerm();
+#endif
         rtR0TermNative();
     }
     return rc;
@@ -109,6 +125,9 @@ static void rtR0Term(void)
 #ifndef IN_GUEST /* play safe for now */
     rtR0PowerNotificationTerm();
     rtR0MpNotificationTerm();
+#endif
+#ifdef RTR0MEM_WITH_EF_APIS
+    rtR0MemEfTerm();
 #endif
     rtR0TermNative();
 }
@@ -126,6 +145,8 @@ RTR0DECL(void) RTR0Term(void)
     Assert(cNewUsers >= 0);
     if (cNewUsers == 0)
         rtR0Term();
+    else if (cNewUsers < 0)
+        ASMAtomicIncS32(&g_crtR0Users);
 }
 RT_EXPORT_SYMBOL(RTR0Term);
 
@@ -134,7 +155,9 @@ RT_EXPORT_SYMBOL(RTR0Term);
 RTR0DECL(void) RTR0TermForced(void)
 {
     RT_ASSERT_PREEMPTIBLE();
+
     AssertMsg(g_crtR0Users == 1, ("%d\n", g_crtR0Users));
+    ASMAtomicWriteS32(&g_crtR0Users, 0);
 
     rtR0Term();
 }
