@@ -1,4 +1,4 @@
-/* $Id: the-linux-kernel.h 118412 2017-10-17 14:26:02Z bird $ */
+/* $Id: the-linux-kernel.h 133285 2019-09-11 16:44:07Z paulson $ */
 /** @file
  * IPRT - Include all necessary headers for the Linux kernel.
  */
@@ -119,6 +119,9 @@
 # include <linux/cpu.h>
 # include <linux/notifier.h>
 #endif
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(5, 1, 0)
+# include <uapi/linux/mman.h>
+#endif
 /* For the basic additions module */
 #include <linux/pci.h>
 #include <linux/delay.h>
@@ -448,5 +451,13 @@ typedef struct tq_struct    RTR0LNXWORKQUEUEITEM;
 DECLHIDDEN(void) rtR0LnxWorkqueuePush(RTR0LNXWORKQUEUEITEM *pWork, void (*pfnWorker)(RTR0LNXWORKQUEUEITEM *));
 DECLHIDDEN(void) rtR0LnxWorkqueueFlush(void);
 
+/*
+ * Guest Additions changes specific to Red Hat 8.1 and later.
+ */
+#ifdef RHEL_RELEASE_CODE
+# if RHEL_RELEASE_CODE >= RHEL_RELEASE_VERSION(8, 1)
+#  define RHEL_81
+# endif
+#endif
 
 #endif
